//*******************************************************************
//
//  UNIT:     B u m p L i                        06.08.99,  09.09.99
//
//  bump lightning demo
//
//-------------------------------------------------------------------
//
//  StartBumpLightning     start bump lightning demo
//  ResizeBumpLightning    called on new window size
//  BumpLightning          animated true color bump lightning demo
//
//*******************************************************************
unit BumpLi;

interface

function StartBumpLightning (demo: byte): string;
procedure ResizeBumpLightning;
procedure BumpLightning;

//*********************************************************
implementation

uses  SysUtils, Windows, Graphics,
      Graph, Palette, FractMap, Dance, Blur;

const minLI =  33;      // minimum of light intensity
      maxLI = 200;      // maximum of light intensity

var   heightMap:   tBitmap = nil;      // 2^n maps
      colorMap:    tBitmap = nil;

      heightBm:    tBitmap = nil;      // bitmaps
      colorBm:     tBitmap = nil;
      colIntBm:    tBitmap = nil;

      demoNo:      integer =  1;       // demo number
      moveSpeed:   integer = 30;       // moving speed
      moveRadius:  integer = 65;       // moving radius
      lSize:       integer =  8;       // light size
      roughness:   integer = 55;       // map roughness: 1..100

//*********************************************************
function StartBumpLightning (demo: byte): string;
begin
  demoNo := demo;
  result := 'Bump Lightning ' + IntToStr(demo);
  SetGraphMode (pf24bit);
  GenerateMap (roughness);
  ResizeBumpLightning;
end;

//*********************************************************
// create and calculate color intensity table
//---------------------------------------------------------
procedure ColorIntensityMap;
var   xi,yi: longint;
      palCol: tRGB;
begin
  // create and / or resize of intensity map
  if colIntBm = nil
  then begin
    colIntBm := tBitmap.Create;
    colIntBm.PixelFormat := pfDevice;
  end;
  colIntBm.Width  := 256;
  colIntBm.Height := 256;

  // calculate values
  for yi := 0 to 255
  do begin
    palCol := cPal[yi];
    for xi := 0 to 255
    do with palCol
    do PutPixel (xi,yi, RGB(rb * xi shr 8
                           ,gb * xi shr 8
                           ,bb * xi shr 8));
  end;
end;
//*********************************************************
// create / resize height and color bitmaps
//---------------------------------------------------------
procedure ResizeBumpLightning;
begin
  // create height map with window size
  if heightBm = nil
  then begin
    heightBm := tBitmap.Create;
    heightBm.PixelFormat := pf8Bit;
  end;
  heightBm.Width  := pageX;
  heightBm.Height := pageY;
  SetBitmapPalette (heightBm, cPal);
//  SetIntensityPalette (heightBm);
  DrawTilesBackground (0,0, heightMap, heightBm);

  // create color map with window size
  if colorBm = nil
  then begin
    colorBm := tBitmap.Create;
    colorBm.PixelFormat := pf8Bit;
  end;
  colorBm.Width  := pageX;
  colorBm.Height := pageY;
  SetBitmapPalette (colorBm, cPal);
  DrawTilesBackground (0,0, colorMap, colorBm);

  if demoNo >= 3
  then ResizeLightningMap;
end;
//*********************************************************
// Bump-Lightning Animation 1
// uses only light direction
// input:      sm: height bitmap
//             cm: color bitmap
//             dm: destination bitmap
//---------------------------------------------------------
procedure BL1Anim (var sm,cm,dm: tBitmap);
var
    lx,ly,                   // light position
    xi,yi,                   // pixel position
    li: longint;             // light intensity
    hLeft,hMid,hRight: byte; // actual line heights
    h1, h2, h3: ^byte;       // height line pointer
    rgbPtr: ^byte;           // drawing pointer
    cp: ^byte;               // color pointer
    color: tRGB;             // actual pixel color
begin
  // calculate light position
  lx := round(cos(picNo * moveSpeed / 256) * moveRadius);
  ly := round(sin(picNo * moveSpeed / 256) * moveRadius);

  h1 := sm.ScanLine[0];
  h2 := h1;
  h3 := sm.ScanLine[1];
  for yi := 0 to GetMaxY-1
  do begin
    hLeft  := h2^;
    hMid   := h2^;
    hRight := h2^;
    rgbPtr  := dm.ScanLine[yi];
    cp      := cm.ScanLine[yi];
    for xi := 0 to GetMaxX
    do begin
      // calculate light intensity
      li := 160 - ly*(h1^ - h3^)
                - lx*(hRight - hLeft);
      if      li >   255 then li := 255
      else if li < minLi then li := minLI;

      // calculate color intensity of the pixel and set it
      color := cPal[cp^];
      rgbPtr^ := (color.bb * li) shr 8;  inc (rgbPtr);
      rgbPtr^ := (color.gb * li) shr 8;  inc (rgbPtr);
      rgbPtr^ := (color.rb * li) shr 8;  inc (rgbPtr);

      inc(h1);               // update source pointer
      inc(h2);
      inc(h3);
      inc(cp);

      hLeft  := hMid;        // shift pixel value
      hMid   := hRight;
      hRight := h2^;
    end;
    h1 := sm.ScanLine[yi];
    h2 := sm.ScanLine[yi+1];
    h3 := sm.ScanLine[yi+2];
  end;
  CopyLine (0,GetMaxY-1, pageX, 0,GetMaxY);
  // draw light position
  // PutPixel (pageX div 2 + lx, pageY div 2 - ly, clYellow);
end;

//*********************************************************
// Bump-Lightning Animation 2
// uses online calculated light intensity
// input:      sm: height bitmap
//             cm: color bitmap
//             dm: destination bitmap
//---------------------------------------------------------
procedure BL2Anim (var sm,cm,dm: tBitmap);
var
    lx,ly,                   // light position
    vx,vy,                   // distance to light
    xi,yi,                   // pixel position
    li: longint;             // light intensity
    hLeft,hMid,hRight: byte; // actual line heights
    h1, h2, h3: ^byte;       // height line pointer
    rgbPtr: ^byte;           // drawing pointer
    cp: ^byte;               // color pointer
    color: tRGB;             // actual pixel color
begin
  // calculate light position
  lx := pageX div 2 + round(cos(picNo*moveSpeed / 256) * moveRadius);
  ly := pageY div 2 + round(sin(picNo*moveSpeed / 200) * moveRadius);

  vy := -ly;                 // init y-distance to light
  h1 := sm.ScanLine[0];      // init height line pointer
  h2 := h1;
  h3 := sm.ScanLine[1];
  for yi := 0 to GetMaxY-1
  do begin
    vx := -lx;
    hLeft  := h2^;
    hMid   := h2^;
    hRight := h2^;
    rgbPtr  := dm.ScanLine[yi];
    cp      := cm.ScanLine[yi];
    for xi := 0 to GetMaxX
    do begin
      // calculate light intensity
      li := 255 - (abs(vy + (h1^ - h3^) shl 4)
                  +abs(vx + (hRight - hLeft) shl 4)) shl 3 div lSize;
      if li < minLi then li := minLI;

      // calculate color intensity of the pixel and set it
      color := cPal[cp^];
      rgbPtr^ := (color.bb * li) shr 8;  inc (rgbPtr);
      rgbPtr^ := (color.gb * li) shr 8;  inc (rgbPtr);
      rgbPtr^ := (color.rb * li) shr 8;  inc (rgbPtr);

      inc(vx);               // update y-distance
      inc(h1);               // update source pointer
      inc(h2);
      inc(h3);
      inc(cp);

      hLeft  := hMid;        // shift pixel value
      hMid   := hRight;
      hRight := h2^;
    end;
    inc(vy);                 // update y-distance
    h1 := sm.ScanLine[yi];
    h2 := sm.ScanLine[yi+1];
    h3 := sm.ScanLine[yi+2];
  end;
  CopyLine (0,GetMaxY-1, pageX, 0,GetMaxY);
  // draw light position
  // PutPixel (lx, ly, clOlive);
end;
//*********************************************************
// Bump-Lightning Animation 3
// uses precalculated intensity light map
// input:      sm: height bitmap
//             cm: color bitmap
//             dm: destination bitmap
//---------------------------------------------------------
procedure BL3Anim (var sm,cm,dm: tBitmap);
var
    lx,ly,                   // light position
    xi,yi,                   // pixel position
    li: longint;             // light intensity
    hLeft,hMid,hRight: byte; // actual line heights
    h1, h2, h3: ^byte;       // height line pointer
    rgbPtr: ^byte;           // drawing pointer
    cp: ^byte;               // color pointer
    lp: ^byte;               // light intensity pointer
    color: tRGB;             // actual pixel color
begin
  // calculate light position
  lx := pageX div 2 + round(cos(picNo*moveSpeed / 256) * moveRadius);
  ly := pageY div 2 + round(sin(picNo*moveSpeed / 201) * moveRadius);

  h1 := sm.ScanLine[0];      // init height line pointer
  h2 := h1;
  h3 := sm.ScanLine[1];
  for yi := 0 to GetMaxY-1
  do begin
    hLeft  := h2^;
    hMid   := h2^;
    hRight := h2^;
    rgbPtr := dm.ScanLine[yi];
    cp     := cm.ScanLine[yi];
    lp := lpp (yi + pageY - ly);
    inc (lp, pageX-lx);
    for xi := 0 to GetMaxX
    do begin
      // calculate light intensity
      li := lp^ * lSize shr 2 - 16 * ((h1^ - h3^)
                       +(hRight - hLeft));
      if      li >   255 then li := 255
      else if li < minLi then li := minLI;

      // calculate color intensity of the pixel and set it
      color := cPal[cp^];
      rgbPtr^ := (color.bb * li) shr 8;  inc (rgbPtr);
      rgbPtr^ := (color.gb * li) shr 8;  inc (rgbPtr);
      rgbPtr^ := (color.rb * li) shr 8;  inc (rgbPtr);

      inc(h1);               // update source pointer
      inc(h2);
      inc(h3);
      inc(cp);
      inc(lp);

      hLeft  := hMid;        // shift pixel value
      hMid   := hRight;
      hRight := h2^;
    end;
    h1 := sm.ScanLine[yi];
    h2 := sm.ScanLine[yi+1];
    h3 := sm.ScanLine[yi+2];
  end;
  CopyLine (0,GetMaxY-1, pageX, 0,GetMaxY);
end;
//*********************************************************
// create new maps
//---------------------------------------------------------
procedure NewMap (dr: integer);
begin
  roughness := roughness + dr;
  GenerateMap (roughness);
  ResizeBumpLightning;
// DrawMap (0,0,7);
  MarkTime;
end;
//*********************************************************
procedure Bluring;
begin
  Blur3H (heightBm, heightBm, 0,heightBM.height-1);
end;
//*********************************************************
// display transparent shadowed help text
//---------------------------------------------------------
procedure DisplayHelp;
var   h: word;
begin
  TextColor (clWhite);
  h := TextHeight;
  ShadowTextXY (10,1*h, '<ret> create new map');
  ShadowTextXY (10,2*h, ' i/j   3 moving speed ' + IntToStr(moveSpeed));
  ShadowTextXY (10,3*h, ' o/k  moving radius ' + IntToStr(moveRadius));
  ShadowTextXY (10,4*h, ' r/w  read/write fractal map');
  ShadowTextXY (10,5*h, ' P/L  fractal dimension ' + IntToStr(roughness));
  ShadowTextXY (10,6*h, ' +/-  light size ' + IntToStr(lSize));
end;
//*********************************************************
// draw one picture and handle keyboard input
//---------------------------------------------------------
procedure BumpLightning;
begin
  case demoNo of
    1: BL1Anim (heightBm, colorBm, bgBitmap);
    2: BL2Anim (heightBm, colorBm, bgBitmap);
    3: BL3Anim (heightBm, colorBm, bgBitmap);
  end;

  if animHelp then DisplayHelp;
  case cb of
    #13: NewMap (0);
    'i': inc(moveSpeed);
    'j': dec(moveSpeed);
    'o': inc(moveRadius,2);
    'k': dec(moveRadius,2);
    'r': begin LoadMap ('BL'); ResizeBumpLightning; end;
    'w': SaveMap ('BL');
    'P': NewMap (+1);
    'L': NewMap (-1);
    '+': inc(lSize);
    '-': if lSize > 1 then dec(lSize);
    'b': Bluring;
  end;
  cb := #0;
end;
//*********************************************************
initialization
  CreateMap (heightMap, colorMap);
  ColorIntensityMap;

finalization
  DeleteMap (heightMap, colorMap);

end.

