//*******************************************************************
//
//  UNIT:     D r a w R G B                      02.06.99,  31.08.99
//
//-------------------------------------------------------------------
//
//  DrawRGB          draw color rectangle
//  GouraudHLine     draw horizontal line with interpolated colors
//  MixColor         mit 2 colors with given intesity together
//
//*******************************************************************
unit DrawRGB;

interface

uses Graphics, Graph;

function StartRGBColors: string;
procedure DrawRGBColors (x1,y1,xdim,ydim: integer);
procedure GouraudHLine (x1,x2,y: integer; c1,c2: tColor);
procedure MixColor (var mix, c1,c2: tColor; int: byte);

implementation

//*********************************************************
// example of fast pixel drawing using tBitmap.ScanLine[..]
//---------------------------------------------------------
function StartRGBColors: string;
begin
  result := 'fast pixel demo';
  SetGraphMode (pf24bit);
  DrawRGBColors (0,0,pageX,pageY);
end;
//*********************************************************
// Mix colors c1 and c2 with intensity int and return
// this color value. intensity range: 0 .. 255
// calculation:  mix = (c1 * (256-int) + c2 * int) div 256
//---------------------------------------------------------
procedure MixColor (var mix, c1,c2: tColor; int: byte);
var  r1,r2: longint;
begin
  r1 := 256 - int;
  r2 := int;
  mix := ((c1 shr 16 and $FF) * r1
        + (c2 shr 16 and $FF) * r2) div 256 shl 16
        +((c1 shr  8 and $FF) * r1
        + (c2 shr  8 and $FF) * r2) div 256 shl 8
        +((c1        and $FF) * r1
        + (c2        and $FF) * r2) div 256;
end;
//*********************************************************
// draw horizontal line from x1/y to x2/y with interpolated
// color values with start color c1 and end color c2.
//---------------------------------------------------------
procedure GouraudHLine (x1,x2,y: integer; c1,c2: tColor);

const mf = 256;
var   r,g,b, dr,dg,db, dx,xi: longint;
      rgbPtr: ^byte;
begin
  dx := (x2 - x1) * mf;
  if dx <= 0 then exit;   { ===> }
  if (y < 0) or (y > GetMaxY) then exit;
  r := (c1        and $FF) * mf;   dr := ((c2        and $FF) * mf - r) * mf div dx;
  g := (c1 shr 8  and $FF) * mf;   dg := ((c2 shr  8 and $FF) * mf - g) * mf div dx;
  b := (c1 shr 16 and $FF) * mf;   db := ((c2 shr 16 and $FF) * mf - b) * mf div dx;
  rgbPtr := GetPixelPointer (x1,y);
  for xi := x1 to x2
  do begin
    rgbPtr^ := hi(b);   inc (b,db);   inc (rgbPtr);
    rgbPtr^ := hi(g);   inc (g,dg);   inc (rgbPtr);
    rgbPtr^ := hi(r);   inc (r,dr);   inc (rgbPtr);
  end;
end;
//*********************************************************
// draw a filled gouraud rectangle (x1,y1 .. x2,y2)
// with interpolated RGB-Values (c1-c2 .. c3-c4):
//    c1+========+c2
//      |        |
//   cy1+..+cx...+cy2
//      |        |
//    c3+========+c4
//---------------------------------------------------------
procedure GouraudRectangle (x1,y1, x2,y2: integer;
                            c1,c2,c3,c4: tColor);
var  yi,yp, ydim: longint;
     yint: byte;
     c13,c24: tColor;
begin
  ydim := y2 - y1 + 1;
  for yi := 0 to ydim - 1
  do begin
    yp := y1 + yi;
    yint := byte(longint(yi * 255) div ydim);
    MixColor (c13, c1,c3, yint);
    MixColor (c24, c2,c4, yint);
    GouraudHLine (x1,x2,yp, c13,c24);
  end;
end;
//*********************************************************
// draw color table in a rectangle (xdim*ydim)
// with rgb values seen in the table at position (x1,y1).
//
// x1/y1
//   +================== B l a c k ==================+
//   |       |       |       |       |       |       |
//   |       |       |       |       |       |       |
// Magenta--RED----Yellow--GREEN---Cyan----BLUE--Magenta
//   |       |       |       |       |       |       |
//   |       |       |       |       |       |       |
//   +================== W h i t e ==================+
//---------------------------------------------------------
procedure DrawRGBColors (x1,y1,xdim,ydim: integer);

const colors: array[0..6] of tColor
      = (clMagenta, clLightRed, clYellow
        ,clLightGreen
        ,clLightCyan, clLightBlue, clLightMagenta);

var   x2,x3,y2,y3,ri: word;   c1,c2: tColor;  {s1,s2: string;}

begin
  y2 := y1 + ydim div 2;
  y3 := y1 + ydim - 1;
  x2 := x1;
  for ri := 0 to 5
  do begin
    c1 := colors[ri];
    c2 := colors[ri+1];
{
s1 := ColorToString(c1) + '=$' + IntToHex (ColorToRGB(c1),8);
s2 := ColorToString(c2) + '=$' + IntToHex (ColorToRGB(c2),8);
}
    x3 := x1 + (ri+1)*xdim div 6;
    GouraudRectangle (x2, y1,   x3-1,y2
                     ,clBlack, clBlack, c1,c2);    {black .. rgb}
    GouraudRectangle (x2, y2+1, x3-1,y3
                     ,c1,c2, clWhite, clWhite);    {rgb .. white}
    x2 := x3;
  end;
end;

end.

