//*******************************************************************
//
//  UNIT:     M a g n i f y                      08.09.99,  09.09.99
//
//  animated sizeable lens which magnifies background picture
//
//-------------------------------------------------------------------
//
//  StartMagnify           start magnify demo
//  ResizeMagnify          called on new window size
//  AnimateMagnify         animate magnify demo
//
//*******************************************************************
unit Magnify;

interface

function StartMagnify: string;
procedure ResizeMagnify;
procedure AnimateMagnify;

//*********************************************************
implementation

uses  SysUtils, Graphics,
      GrAnim, Graph;

//=========================================================
// Background picture stretching
//=========================================================

const picName = 'POV_Fish.JPG';

var   origBm: tBitmap = nil;   // original loaded background picture
      backBm: tBitmap = nil;   // transformed background picture

      loadError: string[40] = '';

//---------------------------------------------------------
procedure InitBGDemo;
begin
  // load background picture
  if not LoadBitmap (origBm, picName)
  then loadError := 'loading error picture ' + picName
  else if origBm = nil
  then loadError := 'creation error picture' + picName;
end;
//---------------------------------------------------------
procedure ResizeBGDemo;
begin
  if loadError <> ''
  then OutTextXY (5,5, loadError)
  else animWindow.SetBackgroundPicture (origBm);
end;
//---------------------------------------------------------
function StartBGDemo: string;
begin
  result := 'background picture stretching';
  SetGraphMode (pf16Bit);
  ResizeBGDemo;
end;

//=========================================================
// Magnify
//=========================================================

var   moveSpeed:   integer =  20;      // moving speed
      moveRadius:  integer =  80;      // moving radius

      lensRadius:  integer =  50;      // lens radius
      sphRadius:   integer =  60;      // lens sphere radius
      focDist:     integer =  80;      // distance from plane to focus point

      lx:          longint = 0;        // lens left top position
      ly:          longint = 0;

const maxMMI  = 255;                   // max. magnify map index (2^n)
      magSize = maxMMI+1;              // magnify map size

var   magXMap: tBitmap = nil;          // magnify X deviation map
      magYMap: tBitmap = nil;          // magnify Y deviation map

procedure CalculateMagnify; forward;

//*********************************************************
procedure ResetParameter;
begin
  moveSpeed   :=  20;
  moveRadius  :=  80;
  lensRadius  :=  50;
  sphRadius   :=  60;
  focDist     :=  80;
  CalculateMagnify;
end;
//*********************************************************
// create magnify maps
//---------------------------------------------------------
procedure InitMagnify;

begin
  // create magnify X map
  if magXMap = nil
  then begin
    magXMap := tBitmap.Create;
    magXMap.PixelFormat := pf16Bit;
    magXMap.Width  := magSize;
    magXMap.Height := magSize;
  end;

  // create magnify Y map
  if magYMap = nil
  then begin
    magYMap := tBitmap.Create;
    magYMap.PixelFormat := pf16Bit;
    magYMap.Width  := magSize;
    magYMap.Height := magSize;
  end;
end;
//*********************************************************
function StartMagnify: string;
begin
  result := 'magnify sizeable background';
  SetGraphMode (pf16bit);
  ResizeMagnify;
end;
//*********************************************************
// create / resize background bitmap
//---------------------------------------------------------
procedure ResizeMagnify;

begin
  ResizeBGDemo;

  // create and / or copy transformed background picture
  if backBm = nil
  then begin
    backBm := tBitmap.Create;
    backBm.PixelFormat := pf16Bit;
  end;
  backBm.Width  := bgBitmap.Width;
  backBm.Height := bgBitmap.Height;
  CopyBitmap (bgBitmap, backBm);
end;
//*********************************************************
// calculate lens magnification values
//---------------------------------------------------------
procedure CalculateMagnify;
var
  xi,yi,z  : Integer;
  ux,uy    : Integer;         // distance to center point
  xq,yq    : longint;         // x/y square values
  rq,sq    : longint;         // radius square values
  xp,yp    : ^smallInt;       // pointer to correction value
begin
  rq := lensRadius * lensRadius;
  sq := sphRadius * sphRadius;
  for yi := 0 to maxMMI
  do begin
    xp := magXMap.scanline[yi];
    yp := magYMap.scanline[yi];
    uy := yi - magSize div 2;
    yq := uy * uy;
    for xi := 0 to maxMMI
    do begin
      ux := xi - magSize div 2;
      xq := ux * ux;
      if xq + yq >= rq - 10
      then begin
        xp^ := 0;
        yp^ := 0;
      end
      else begin
        z   := round (sqrt (sq - xq - yq));
        xp^ := round ((focDist-z) * ux/z);
        yp^ := round ((focDist-z) * uy/z);
      end;
      inc (xp);
      inc (yp);
    end;
  end;
end;
//*********************************************************
procedure DrawMagnifyPicture;

var
  mx,my,                // background picture dimension
  dx,dy,                // destination position
  xi,yi,                // pixel position
  xn,yn: longint;       // diversion values
  sp: pWordArray;       // source pointer
  xp,yp: ^smallInt;     // magnify values
  dp: ^word;            // destination pointer

begin
  // restore lens rectangle
  CopyRect (backBm, lx,ly,lx,ly, magSize,magSize);

  // calculate light position
  lx := pageX div 2 + round(cos(picNo*moveSpeed / 256) * moveRadius);
  ly := pageY div 2 + round(sin(picNo*moveSpeed / 200) * moveRadius);
  lx := lx - magSize div 2;
  ly := ly - magSize div 2;

  mx := backBm.Width;
  my := backBm.Height;

  dy := ly;
  for yi := 0 to maxMMI
  do begin
    if dy >= my then break;            // destination line outside ?
    if dy >= 0                         // destination line inside ?
    then begin
      dx := lx;
      dp := bgBitmap.scanline[dy];     // destination pixel pointer
      inc(dp,dx);
      xp := magXMap.scanline[yi];      // deviation value pointer
      yp := magYMap.scanline[yi];
      for xi := 0 to maxMMI
      do begin
        if (dx > 0) and (dx < mx)      // drawing position inside ?
        then if (xp^ <> 0) or (yp^ <> 0)
        then begin
          yn := dy + yp^;              // get refracted y position
          if yn < 0 then yn := 0
          else if yn >= my then yn := my -1;

          xn := dx + xp^;              // get refracted x position
          if xn < 0 then xn := 0
          else if xn >= mx then xn := mx - 1;

          sp := backBm.scanline[yn];   // get source line pointer
          dp^ := sp^[xn];              // draw pixel
        end;
        inc (xp);
        inc (yp);
        inc (dp);
        inc (dx);
      end;
    end;
    inc (dy);
  end;
end;
//*********************************************************
procedure ChangeFocusDistance (dv: integer);
begin
  focDist := focDist + dv;
  CalculateMagnify;
end;
//*********************************************************
procedure ChangeSphereRadius (dr: integer);
begin
  sphRadius := sphRadius + dr;
  if sphRadius < lensRadius then sphRadius := lensRadius;
  CalculateMagnify;
end;
//*********************************************************
procedure ChangeLensRadius (dr: integer);
begin
  lensRadius := lensRadius + dr;
  if      lensRadius < 10 then lensRadius := 10
  else if lensRadius > maxMMI div 2
  then    lensRadius := maxMMI div 2;
  ChangeSphereRadius (0);              // correct sphere radius
  CalculateMagnify;
end;
//*********************************************************
// display transparent shadowed help text
//---------------------------------------------------------
procedure DisplayHelp;
var   h: word;
begin
  TextColor (clWhite);
  h := TextHeight;
  ShadowTextXY (10,1*h, '<ret> restart');
  ShadowTextXY (10,2*h, '  i/j   moving speed ' + IntToStr(moveSpeed));
  ShadowTextXY (10,3*h, ' o/k  moving radius '  + IntToStr(moveRadius));
  ShadowTextXY (10,4*h, ' P/L  focus distance ' + IntToStr(focDist));
  ShadowTextXY (10,5*h, ' +/-  lens radius '    + IntToStr(lensRadius));
  ShadowTextXY (10,6*h, ' ,/.  sphere radius '  + IntToStr(sphRadius));
end;
//*********************************************************
// calculate magnification values
//---------------------------------------------------------
procedure AnimateMagnify;

begin
  DrawMagnifyPicture;

  if animHelp then DisplayHelp;
  case cb of
    #13: ResetParameter;
    'i': inc(moveSpeed);
    'j': dec(moveSpeed);
    'o': inc(moveRadius,2);
    'k': dec(moveRadius,2);
    'P': ChangeFocusDistance (+1);
    'L': ChangeFocusDistance (-1);
    '+': ChangeLensRadius (+1);
    '-': ChangeLensRadius (-1);
    '.': ChangeSphereRadius (+1);
    ',': ChangeSphereRadius (-1);
  end;
end;
//*********************************************************
initialization
  InitBGDemo;
  InitMagnify;
  CalculateMagnify;

finalization
  origBm.Free;
end.

