//*******************************************************************
//
//  UNIT:     S h e l l y                        29.06.98,  31.08.99
//
//  simulation of shell pattern generation with Delphi 4
//
//  feature: creating new rules on floating point exception
//
//  References:
//
//   The Algorithmic Beauty of Sea Shells, Hans Meinhard
//   Springer 95, Basic
//
//   Wie Schnecken sich in Schale werfen, Hans Meinhard
//   Springer 97, Basic
//
//-------------------------------------------------------------------
//
//  StartShelly         start shelly demo
//  ResizeShelly        called on new window size
//  Shelly1             pattern generation 1
//
//*******************************************************************
unit Shelly;

interface

function StartShelly: string;
procedure ResizeShelly;
procedure Shelly1;

implementation

uses SysUtils, Graphics, Graph, Palette;

{=========================================================}
{  P a l e t t e
{=========================================================}

const baseColor = 20;
      colNo: longint = 0;

var   f1,f2,f3: integer;

{*********************************************************}
procedure InitColors;
begin
  colNo := random(5);
  f1 := 21+random(15);
  f2 := 22+random(14);
  f3 := 23+random(13);
end;
{*********************************************************}
procedure AnimColors;
var   ci: byte;   pal: tPalette;
begin
  InitPalette;
  GetAllPalette (pal);
  for ci := baseColor to 255
  do with pal[ci]
  do begin
    rb := 128+trunc((cos((colNo+ci)/f1))*127);
    gb := 128+trunc((cos((colNo+ci)/f2))*127);
    bb := 128+trunc((cos((colNo+ci)/f3))*127);
  end;
  SetAllPalette (pal);
  inc(colNo,7);
end;

{=========================================================}
{  S h e l l y
{=========================================================}

const yl = 40;
      maxL = 1200;
      viewChart: boolean = false;

type  float = double; {extended;}
      fArray = array[0..maxL] of float;

var   fa,fb,fz: fArray;

type  tShelly = record
        col: tColor;    // chart color
        d: float;       // diffusion coefficient < 0.4
        r: float;       // Abbaurate
        b: float;       // Basale
        s: float;       // Sttigung
        c: float;       // Kopplung
        a: float;       // Anfangskonzentration
        g: float;       // Generelle Anfangskonzentration
      end;

var   da,ra,sa,dac,     // activator
      db,rb,sb,dbc,     // inhibitor
      rc,cc: float;

      wl: integer = 2000;    // wanted display line
      dl: integer = 0;       // actual display line
      kp: longint = 5;       // iterations

{*********************************************************}
function RandomValue(minV, range: float): float;
begin
  result := minV + random(65000)*range / 65000;
end;
{*********************************************************}
procedure NewRules;
begin
{
  da := 0.15;
  ra := 0.1;
  sa := 0.25;
  db := 0.01;
  rb := 0.014;
  sb := 0.1;
  rc := 0.1;
  cc := 0.5;
}
  kp := 3+random(6);
  da := RandomValue (0.15, 0.2);   { < 0.4}
  ra := RandomValue (0.1, 0.02);
  sa := RandomValue (0.25, 0.1);

  db := RandomValue (0.01,  0.2);   { < 0.4}
  rb := RandomValue (0.01, 0.01);
  sb := RandomValue (0.1 , 0.05);

  rc := RandomValue (0.15, 0.1);
  cc := RandomValue (0.3, 0.4);
  dac := 1 - ra - 2*da;
  dbc := 1 - rb - 2*db;
end;
{*********************************************************}
procedure CreateValues;
var   xi,ci: integer;
begin
  Randomize;
  FillChar (fa,sizeof(fArray),0);
  FillChar (fb,sizeof(fArray),0);
  FillChar (fz,sizeof(fArray),0);
  for xi := 0 to maxL
  do begin
    fb[xi] := RandomValue(0.01, 0.1);
    fz[xi] := ra * RandomValue(0.96, 0.08);
  end;
  ci := 0;
  for xi := 1 to maxL
  do begin
    ci := ci + 5 + random(40);
    if ci > maxL then break;
    fa[ci] := RandomValue(0.1, 9.9);
  end;
end;
{*********************************************************}
procedure InitShelly;
begin
  InitColors;
  AnimColors;
  CreateValues;
  NewRules;
  wl := 2000;      // wanted display line
  dl := 0;         // active display line
  ResizeShelly;
end;
//*********************************************************
procedure ResizeShelly;
begin
  if dl > GetMaxY then dl := GetMaxY;
  if wl > GetMaxY then wl := GetMaxY;
  if GetMaxY < 100 then viewChart := false;
  if viewChart
  then wl := GetMaxY - yl
  else wl := GetMaxY;
  SetBkColor (black);
  Bar (0,dl, GetMaxX,GetMaxY);
end;
{*********************************************************}
procedure RandomValues;
var   xi,ci: integer;
begin
  ci := 0;
  for xi := 1 to maxL
  do begin
    ci := ci+random(10);
    if ci > maxL then break;
    fa[ci] := RandomValue (0.1, 0.3);
  end;
end;
{*********************************************************}
procedure CheckValues;
var   xi: integer;
begin
  for xi := 1 to GetMaxX
  do if fa[xi] > 0.1 then exit;
  CreateValues;
end;
{*********************************************************}
procedure DrawChart;
var   xi, yi: integer;
begin
  yi := dl + 1;
  SetBkColor (black);
  Bar (0,yi, GetMaxX,yi+yl);

  SetColor (LightRed);
  MoveTo (0, yi + trunc(30*ln(1+fa[0])));
  for xi := 1 to pageX
  do LineTo (xi, yi + trunc(30*ln(1+fa[xi])));

  SetColor (LightGreen);
  MoveTo (0, yi + trunc(30*ln(1+fb[0])));
  for xi := 1 to pageX
  do LineTo (xi, yi + trunc(30*ln(1+fb[xi])));
end;
{*********************************************************}
procedure RestartShelly;
begin
  NewRules;
  CreateValues;
//  beep;
end;
{*********************************************************}
procedure Run;
var   ii, xi, kx: integer;
      linePtr: pByteArray;
      af,bf, a2, aq, bsa, a1,b1: float;
begin
  try
    {calculate new line}
    kx := GetMaxX-1;
    for ii := 1 to kp
    do begin
      a1 := fa[0];
      b1 := fb[0];
      bsa := 0;
      for xi := 1 to kx
      do begin
        af := fa[xi];   a2 := af*af;
        bf := fb[xi];
        aq := fz[xi]*a2/(1+sa*a2);
        fa[xi] := af * dac + da * (a1 + fa[xi+1]) + aq/(sb+bf);
        fb[xi] := bf * dbc + db * (b1 + fb[xi+1]) + aq;
        bsa := bsa + rc * af;
        a1 := af;
        b1 := bf;
        // prevent floation point exceptions
        if fa[xi] < 0.00001 then fa[xi] := 0.00001;
        if fb[xi] < 0.00001 then fb[xi] := 0.00001;
      end;
      cc := cc*(1-rc) + bsa/kx;
      dbc := 1 - 2*db - rb/cc;
      fa[GetMaxX] := fa[1];   {make endless pattern}
      fb[GetMaxX] := fb[1];
      fa[0] := fa[kx];
      fb[0] := fb[kx];
    end;

    {draw new line}
    linePtr := GetLinePointer(dl);
    for xi := 0 to GetMaxX
    do linePtr^[xi] := byte(baseColor+trunc(200*ln(1+fa[xi])));

    if viewChart then DrawChart;

    if dl < wl
    then inc (dl)
    else begin
      if dl > wl
      then begin
        CopyRect (bgBitmap, 0,1, 0,0, pageX,dl);  // move picture 1 row up
        dec (dl);
      end;
      CopyRect (bgBitmap, 0,1, 0,0, pageX,dl);    // move picture 1 row up
    end;

    if picNo and $3F = 0 then CheckValues;

  except
    on EMathError     do RestartShelly;
    on EIntError      do RestartShelly;
  end;
end;

{*********************************************************}
procedure ToggleChart;
begin
  viewChart := not viewChart;
  MarkTime;
  ResizeShelly;
end;
{*********************************************************}
procedure DisplayHelp;
var   h: word;
begin
  TextColor (clWhite);
  h := TextHeight;
  ShadowTextXY (10,1*h, '  n    new rules');
  ShadowTextXY (10,2*h, '  o    chart on/off');
  ShadowTextXY (10,3*h, '  r    cycle colors');
  ShadowTextXY (10,4*h, '  v    random values');
  ShadowTextXY (10,5*h, ' +/-  change iteration deep');
  ShadowTextXY (10,6*h, '<ret> start again');
end;
{*********************************************************}
procedure Shelly1;
begin
  Run;
  if animHelp then DisplayHelp;
  case cb of
    #13: InitShelly;
    '+': inc(kp);
    '-': if kp >1 then dec(kp);
    'o': ToggleChart;
    'r': AnimColors;
    'v': RandomValues;
    'n': NewRules;
  end;
  cb := #0;
end;
//*********************************************************
function StartShelly: string;
begin
  result := 'shell pattern generator';
  SetGraphMode (pf8Bit);

  InitShelly;
end;

end.

